﻿using LibVLCSharp.Shared;
using LibVLCSharp.WPF;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace PlayRtsp2
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        int WINDOW_HEADER_HEIGHT = 30;
        int FRAME_WIDTH = 3200;
        int FRAME_HEIGHT = 1800;
        float scale = 1;
        Point startPoint = new Point(0, 0);
        Thickness marginOrigin;
        bool isMouseDown = false;
        private LibVLC _libVLC;
        private MediaPlayer _mediaPlayer;
        const string VIDEO_URL = "rtsp://admin:abcd1234@sanbongtc.cameraddns.net:554/Streaming/Channels/101";
        Point topLeft;
        Point bottomRight;

        public MainWindow()
        {
            Core.Initialize();
            InitializeComponent();
            _libVLC = new LibVLC();
            _mediaPlayer = new MediaPlayer(_libVLC);
            videoView.MediaPlayer = _mediaPlayer;
            _mediaPlayer.Buffering += (s, e) =>
            {
                ;
            };

            var media = new Media(_libVLC, VIDEO_URL, FromType.FromLocation);
            _mediaPlayer.Play(media);


            videoView.MediaPlayer.EnableMouseInput = true;
            videoView.MediaPlayer.EnableKeyInput = true;
            videoView.MediaPlayer.EnableHardwareDecoding = true;
            this.Show();
            ;

        }

        public void ShowControl()
        {
            Window overlayWindow = new Window();
            // Tạo cửa sổ chính (cửa sổ cũ)
            Window mainWindow = Application.Current.MainWindow;
            // Đặt thuộc tính Owner để chỉ định cửa sổ chính
            // Đặt thuộc tính để loại bỏ border và background của Window
            overlayWindow.WindowStyle = WindowStyle.None;
            overlayWindow.AllowsTransparency = true;
            overlayWindow.Background = System.Windows.Media.Brushes.Transparent;
            overlayWindow.Width = mainWindow.Width;
            overlayWindow.Height = mainWindow.Height - WINDOW_HEADER_HEIGHT;

            // Tạo một Grid trong Window để chứa nội dung của lớp phủ
            Grid overlayGrid = new Grid();
            overlayWindow.Content = overlayGrid;

            overlayGrid.RowDefinitions.Add(new RowDefinition());
            overlayGrid.RowDefinitions.Add(new RowDefinition() {Height=GridLength.Auto });
            DockPanel dockPanel = new DockPanel();
            var butt = new Button() { Content = "maximix" };
            //dockPanel.Children.Add(butt);
            butt.Click += (s, e) =>
            {
                this.WindowState = WindowState.Maximized;
            };
            Grid.SetRow(dockPanel, 1);
            //overlayGrid.Children.Add(dockPanel);
            // Tạo Rectangle làm lớp phủ
            Border overlayRectangle = new Border();
            overlayRectangle.Background = System.Windows.Media.Brushes.Red;
            overlayRectangle.Opacity = 0.01;
            // Hiển thị cửa sổ chính trước
            mainWindow.Show();
            overlayWindow.Left = Left;
            overlayWindow.Top = Top + WINDOW_HEADER_HEIGHT;
            // Gán sự kiện LocationChanged để cập nhật vị trí của overlayWindow khi mainWindow di chuyển
            LocationChanged += (sender, e) =>
                {
                    if (overlayWindow != null && overlayWindow.IsVisible)
                    {
                        // Cập nhật vị trí của overlayWindow dựa trên vị trí của mainWindow
                        overlayWindow.Left = Left;
                        overlayWindow.Top = Top + WINDOW_HEADER_HEIGHT;
                    }
                };

            // Gán sự kiện Closed để đóng overlayWindow khi mainWindow đóng
            Closed += (sender, e) =>
            {
                overlayWindow.Close();
            };

            // Thêm Rectangle vào Grid
            overlayGrid.Children.Add(overlayRectangle);

            overlayWindow.Owner = mainWindow;
            // Hiển thị cửa sổ
            overlayWindow.ShowInTaskbar = false;
            overlayWindow.Show();
            this.SizeChanged += (s, e) =>
            {
                overlayWindow.Width = e.NewSize.Width;
                overlayWindow.Height = e.NewSize.Height - WINDOW_HEADER_HEIGHT;
                scale = GetScale(FRAME_WIDTH, FRAME_HEIGHT);
                videoView.MediaPlayer.Scale = scale;
                videoView.Margin = new Thickness(0);
                CaculatePoint();
            };
            overlayWindow.MouseWheel += (sender, e) =>
            {
                int delta = e.Delta;

                if (delta > 0)
                {
                    scale += (float)0.04;
                }
                else
                {
                    scale -= (float)0.04;
                }
                var size = GetVideoSize();
                if (size.width <= ActualWidth && size.height <= ActualHeight - SystemParameters.CaptionHeight)
                {
                    scale = GetScale(FRAME_WIDTH, FRAME_HEIGHT);
                    videoView.MediaPlayer.Scale = scale;
                    videoView.Margin = new Thickness(0);
                    var size2 = GetVideoSize();
                    this.Title = $"{size2.width}x{size2.height} | {topLeft.X}-{topLeft.Y}";
                CaculatePoint();
                    return;
                }
                videoView.MediaPlayer.Scale = scale;
                CaculatePoint();

                this.Title = $"{size.width}x{size.height} | {topLeft.X}-{topLeft.Y}";
            };
            overlayWindow.MouseDown += (sender, e) =>
            {
                isMouseDown = true;
                startPoint = e.GetPosition(overlayRectangle);
                marginOrigin = videoView.Margin;
                Mouse.OverrideCursor = Cursors.ScrollAll;
            };
            overlayWindow.MouseUp += (sender, e) =>
            {
                isMouseDown = false;
                Mouse.OverrideCursor = null;
            };

            overlayWindow.MouseMove += (sender, e) =>
            {

                if (isMouseDown)
                {
                    var endPoint = e.GetPosition(overlayRectangle);
                    var width = endPoint.X - startPoint.X;
                    var height = endPoint.Y - startPoint.Y;
                    //videoView.MediaPlayer.set
                    // videoView.Width=2000;
                    var x = marginOrigin.Left + width;
                    var y = marginOrigin.Top + height;
                    if (x > topLeft.X)
                        x = topLeft.X;
                    else if (x < bottomRight.X)
                        x = bottomRight.X;
                    if (y > topLeft.Y)
                        y = topLeft.Y;
                    else if (y < bottomRight.Y)
                        y = bottomRight.Y;
                    videoView.Margin = new Thickness(x, y, 0, 0);

                }
            };

        }

        private void Window_Closed_1(object sender, EventArgs e)
        {
            videoView.Dispose();
            _libVLC.Dispose();
        }

        private float GetScale(int frameWidth, int frameHeight)
        {
            // Tính tỉ lệ scale theo chiều rộng và chiều cao
            float scaleX = (float)(ActualWidth) / frameWidth;
            float scaleY = (float)(ActualWidth / ((float)16 / 9)) / frameHeight;

            // Chọn tỉ lệ scale nhỏ hơn để không làm biến đổi hình dạng
            return Math.Min(scaleX, scaleY);
        }
        private (float width, float height) GetVideoSize()
        {

            return (FRAME_WIDTH * (scale), FRAME_HEIGHT * (scale));
        }
        private void CaculatePoint()
        {
            var size = GetVideoSize();
            topLeft = new Point(size.width / 2 - ActualWidth / 2, size.height / 2 - (ActualWidth / ((float)16 / 9)) / 2);
            bottomRight = new Point((size.width / 2 - ActualWidth / 2) * -1, (size.height / 2 - ActualWidth / ((float)16 / 9)/2) * -1);
            topLeft.X = topLeft.X * 1.95;
            topLeft.Y = topLeft.Y * 1.95;
            bottomRight.X = bottomRight.X * 1.95;
            bottomRight.Y = bottomRight.Y * 1.95;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            scale = GetScale(FRAME_WIDTH, FRAME_HEIGHT);
            videoView.MediaPlayer.Scale = scale;
            ShowControl();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            this.WindowState = WindowState.Maximized;
        }
    }
}
